/*                                                                           
* The following structures are nested within the structures used by the    
* system management routines. These structures and sizes were pulled from the AIX
* src tree.
*/                                                                        
#define LVM_MAXLPS      65535       /* max number of logical partitions allowed */
#define LVM_NAMESIZ     64          /* maximum size for the logical volume name */
#define LVM_NUMCOPIES   3           /* max number of copies allowed of a logical partition */
#define LVM_MAXVGS      255
#define LVM_MAXPVS      32
#define LVM_MAXLVS      256
#define AIX_MIN_BLOCK_SIZE 4096
#define VGSA_BT_PV      127
#define NBPI            32
#define TRUE             1
#define OFFSET_CONSTANT     144
#define SLEEP_TIME            0
#define MAXLVS_OFFSET        16
#define PHYS_VOL_OFFSET      34
#define AIX_PVHPP_LENGTH     PHYS_VOL_OFFSET
#define MAX_SECTORS_NAMELIST 32
#define AIX_DEFAULT_MIRRORING 1
#define AIX_FIRST_MIRROR      2
#define AIX_MAX_MIRRORS       3  // AIX defines ALL copies as mirrors - 3 mirrors MAX - 1 orig and 2 copies

#define PSN_LVM_REC      7
#define PSN_VGSA_REC     128
#define PSN_NAMELIST_REC 2065
#define PSN_VGT_TRAILER  135
#define PSN_LVE_REC        1
#define PSN_PPH_OFFSET    17
#define PSN_PVH_INCREMENT 34
#define AIX_SECTOR_SIZE  512
#define MAX_PPENT_SECTOR  16
#define	NAME_LEN		 128	/* don't change!!! */
#define	UUID_LEN		  32    /* don't change!!! */
#define MAX_SECTORS_LV_ENTRIES 16
#define AIX_MIN_MIRROR_POOL    10
#define AIX_MIRROR_POOL_CHANGE 10

#define LV_SET_ACCESS           _IOW ( 0xfe, 0x28, 1)
#define LV_SET_ALLOCATION       _IOW ( 0xfe, 0x29, 1)
#define LV_SET_STATUS           _IOW ( 0xfe, 0x2a, 1)
#define	LV_BMAP		        _IOWR ( 0xfe, 0x30, 1)

#define	LV_ACTIVE            0x01	/* lv_status */
#define	LV_SPINDOWN          0x02	/*     "     */
#define LV_ERROR             0x99   /*     "     */ 

#define	VG_ACTIVE            0x01	/* vg_status */

#define	AIX_LV_READ  	         0x00	/* lv_access */
#define	AIX_LV_WRITE         0x01	/*     "     */
#define EVMS_LV_NEW	         0x10	// volume was created during the current discovery pass
#define EVMS_LV_INCOMPLETE	 0x20	// volume has an incomplete LE map
#define EVMS_LV_INVALID		 0x40	// volume has a memory-corruption problem
#define EVMS_VG_DIRTY		 0x01	// group has had a new PV added during this discovery
#define AIX_VG_INCOMPLETE	 0x20	// volume group is incomplete 


#define LOG_PREFIX		"--AIXlvm: "

// Entries in the list of physical volumes (PV)
// in a volume group (VG)

typedef struct unique_id_s {
    uint32_t  word1;
    uint32_t  word2;
    uint32_t  word3;
    uint32_t  word4;
} unique_id;

typedef struct _partition_list_entry {
	evms_logical_node_t             * logical_node;
	u_int32_t                         pv_number;
	u_int32_t                 block_size;	// bytes
	u_int32_t                             hard_sect_size;	// bytes
	struct _partition_list_entry	* next;

} partition_list_entry_t;

// Table for mapping logical extents (LE) to physical extents (PE)
typedef struct _pe_table_entry {
	partition_list_entry_t	* owning_pv;
	u_int64_t               pe_sector_offset;
} pe_table_entry_t;

// Logical volumes (LV) in a volume group (VG)
typedef struct _aix_logical_volume {
	u_int32_t               lv_number;
	u_int64_t               lv_size;		// Sectors
	u_int32_t               lv_access;		// Flags: LV_READ, LV_WRITE, LV_SNAPSHOT, LV_SNAPSHOT_ORG, EVMS_QUIESCE
	u_int32_t               lv_status;		// Flags: LV_ACTIVE, LV_SPINDOWN
	u_int32_t               lv_minor;		// Device minor number
	u_int32_t           	mirror_copies;	// Do we have mirroring and how many  ?
	u_int32_t           	mirror_number;	// mirror number - which copy is this ?
	u_int32_t           	mirror_iterations;	// Which mirror should we be writing to ?
	u_int32_t               stripes;
	u_int32_t               stripe_size;	    // Sectors
	u_int32_t               stripe_size_shift;  // Number of bits to shift right instead of dividing by stripe_size
	u_int32_t               pe_size;		// Sectors
	u_int32_t               pe_size_shift;	    // Number of bits to shift right instead of dividing by pe_size
	u_int32_t               num_le;			// Number of entries in the le_to_pe_map
	u_int32_t               new_volume;		// Flag to indicate if this volume needs to be exported
	struct _aix_volume_group  * group;		// Pointer back to parent volume group
	unsigned char           name[EVMS_VOLUME_NAME_SIZE+1];	// Dev-tree volume name (eg: /dev/group0/vol0)
	pe_table_entry_t        * le_to_pe_map;	// Mapping of logical to physical extents
	pe_table_entry_t        * le_to_pe_map_mir1;	// Mapping of logical to physical extents for mirror 1
	pe_table_entry_t        * le_to_pe_map_mir2;	// Mapping of logical to physical extents for mirror 2
	evms_logical_node_t     * volume_node;	// Pointer to the parent EVMS node representing this volume

} aix_logical_volume_t;

// Volume groups (VG)
typedef struct _aix_volume_group {
	unique_id       	vg_id;			// volume group number */
	u_int32_t               numpvs;				// Number of PVs found on this VG.
	u_int32_t               numlvs;				// Number of LVs found on this VG.
	u_int32_t               hard_sect_size;			// The largest hard_sect_size and block_size
	u_int32_t               block_size;			    // values of all partitions in this group.
	u_int32_t               flags;			    //
	u_int32_t        	lv_max;			// maximum logical volumes */
	u_int32_t        	pe_size;			    // physical extent size in sectors */
	partition_list_entry_t	* partition_list;	// List of partitions/segments/PVs that make up this VG
	u_int32_t        	partition_count;
	struct _aix_logical_volume      ** volume_list;		// Array of volumes found in this VG.
	struct _aix_volume_group * next;	    // Pointer to the next VG
    u_int32_t                 CleanVGInfo;      // Do we have a clean VG Info to work with ?
	daddr_t                   vgda_psn;	    // Which VGDA we should use
	long                      vgda_len;	    // length of the volume group descriptor area */
	struct _vg_header       * AIXvgh;	    // Pointer to valid data area on disk for the VG
} aix_volume_group_t;

typedef struct _aix_mirror_bh {
    atomic_t                     remaining;
	int     	  	             iteration;     // 'have we finished' count, used from IRQ handlers
	int			                 cmd;
    u_int64_t                    mir_sector1;
    u_int64_t                    mir_sector2;
	struct buffer_head        	*master_bh;
	struct buffer_head	         bh_req;
	struct _aix_mirror_bh       *mirror_bh_list;
	evms_logical_node_t	        *node;	    	// map to evms node (READ only)
	evms_logical_node_t	        *mir_node1;	    	// 
	evms_logical_node_t	        *mir_node2;	    	// 
	eio_t			             eio;
	struct _aix_mirror_bh		*next_r1;	    // next for retry or in free list 
} aix_mirror_bh_t;

typedef struct _timestruc_t 
{
  int tv_sec;
  int tv_nsec;

} timestruc_t;

typedef struct ipl_rec_area
{
    unsigned int      IPL_record_id;    /* This physical volume contains a   */
                                        /* valid IPL record if and only if   */
                                        /* this field contains IPLRECID      */

#define IPLRECID 0xc9c2d4c1             /* Value is EBCIDIC 'IBMA'           */

    char              reserved1[20];
    unsigned int      formatted_cap;    /* Formatted capacity. The number of */
                                        /* sectors available after formatting*/
                                        /* The presence or absence of bad    */
                                        /* blocks does not alter this value. */

    char              last_head;        /* THIS IS DISKETTE INFORMATION      */
                                        /* The number of heads minus 1. Heads*/
                                        /* are number from 0 to last_head.   */

    char              last_sector;      /* THIS IS DISKETTE INFORMATION      */
                                        /* The number of sectors per track.  */
                                        /* Sectors are numbered from 1 to    */
                                        /* last_sector.                      */

    char              reserved2[6];

    unsigned int      boot_code_length; /* Boot code length in sectors. A 0  */
                                        /* value implies no boot code present*/

    unsigned int      boot_code_offset; /* Boot code offset. Must be 0 if no */
                                        /* boot code present, else contains  */
                                        /* byte offset from start of boot    */
                                        /* code to first instruction.        */

    unsigned int      boot_lv_start;    /* Contains the PSN of the start of  */
                                        /* the BLV.                          */

    unsigned int      boot_prg_start;   /* Boot code start. Must be 0 if no  */
                                        /* boot code present, else contains  */
                                        /* the PSN of the start of boot code.*/

    unsigned int      boot_lv_length;   /* BLV length in sectors.            */

    unsigned int      boot_load_add;    /* 512 byte boundary load address for*/
                                        /* boot code.                        */

    char              boot_frag;        /* Boot code fragmentation flag. Must*/
                                        /* be 0 if no fragmentation allowed, */
                                        /* else must be 0x01.                */

    char	      boot_emulation;	/* ROS network emulation flag */
					/* 0x0 => not an emul support image   */
					/* 0x1 => ROS network emulation code  */
					/* 0x2 => AIX code supporting ROS emul*/

    char              reserved3[2];

    ushort            basecn_length;    /* Number of sectors for base        */
                                        /* customization. Normal mode.       */

    ushort            basecs_length;    /* Number of sectors for base        */
                                        /* customization. Service mode.      */

    unsigned int      basecn_start;     /* Starting PSN value for base       */
                                        /* customization. Normal mode.       */

    unsigned int      basecs_start;     /* Starting PSN value for base       */
                                        /* customization. Service mode.      */

    char              reserved4[24];

    unsigned int      ser_code_length;  /* Service code length in sectors.   */
                                        /* A 0 value implies no service code */
                                        /* present.                          */

    unsigned int      ser_code_offset;  /* Service code offset. Must be 0 if */
                                        /* no service code is present, else  */
                                        /* contains byte offset from start of*/
                                        /* service code to first instruction.*/

    unsigned int      ser_lv_start;     /* Contains the PSN of the start of  */
                                        /* the SLV.                          */

    unsigned int      ser_prg_start;    /* Service code start. Must be 0 if  */
                                        /* service code is not present, else */
                                        /* contains the PSN of the start of  */
                                        /* service code.                     */

    unsigned int      ser_lv_length;    /* SLV length in sectors.            */

    unsigned int      ser_load_add;     /* 512 byte boundary load address for*/
                                        /* service code.                     */

    char              ser_frag;         /* Service code fragmentation flag.  */
                                        /* Must be 0 if no fragmentation     */
                                        /* allowed, else must be 0x01.       */

    char	      ser_emulation;	/* ROS network emulation flag */
					/* 0x0 => not an emul support image   */
					/* 0x1 => ROS network emulation code  */
					/* 0x2 => AIX code supporting ROS emul*/

    char              reserved5[2];

    unique_id         pv_id;            /* The unique identifier for this    */
                                        /* physical volume.                  */
    char              dummy[512 - 128 - sizeof(unique_id)];
}AIXIPL_REC, *AIXIPL_REC_PTR;


typedef struct AIXlvm_rec_s
	 /* structure which describes the physical volume LVM record */
       {
       long lvm_id;                        /* LVM id field which identifies whether the PV is a member of a volume group */

#define AIX_LVM_LVMID     0x5F4C564D            /* LVM id field of ASCII "_LVM" */

       unique_id          vg_id;           /* the id of the volume group to which this physical volume belongs */
       long               lvmarea_len;     /* the length of the LVM reserved area */
       long               vgda_len;        /* length of the volume group descriptor area */
       daddr_t            vgda_psn [2];    /* the physical sector numbers of the beginning of the volume group descriptor area copies on this disk */
       daddr_t            reloc_psn;       /* the physical sector number of the beginning of a pool of blocks  */
                                           /* (located at the end of the PV) which are reserved for the relocation of bad blocks */
       long               reloc_len;       /* the length in number of sectors of the pool of bad block relocation blocks */
       short int          pv_num;          /* the physical volume number within the volume group of this physical volume */
       short int          pp_size;         /* the size in bytes for the partition, expressed as a power of 2 (i.e., the partition size is 2 to the power pp_size) */
       long               vgsa_len;        /* length of the volume group status area */
       daddr_t            vgsa_psn [2];    /* the physical sector numbers of the beginning of the volume group status area copies on this disk */
       short int          version;         /* the version number of this volume group descriptor and status area */

#define  LVM_VERSION_1		1              /* first version - AIX 3.0 */
#define  LVM_STRIPE_ENHANCE	2              /* version with striped lv's - AIX 4.1 */
#define  LVM_1024_PPSIZE	3              /* ppsizes of 512 and 1024 */
#define  LVM_GT_1016		4              /* version with support for > 1016 pps/pv */
#define  LVM_MAX_VERSION	LVM_GT_1016    /* max version # */

       char res1 [450];                    /* reserved area */

 } AIXlvm_rec_t;



/*  II.Volume Group Descriptor Area  */

typedef struct _vgsa_area
{
      timestruc_t           b_tmstamp;    /* Beginning timestamp */
      unsigned int          pv_missing [(LVM_MAXPVS + (NBPI -1)) / NBPI];  /* Bit per PV */
      unsigned char         stalepp    [LVM_MAXPVS] [VGSA_BT_PV];
      short                 factor;
      char                  resv[10];     /* Padding */
      timestruc_t           e_tmstamp;    /* Ending timestamp */

} vgsa_area;

typedef struct _vg_header
{
      timestruc_t           vg_timestamp; /* time of last update */
      unique_id             vg_id;        /* unique id for volume group */ 
      short                 numlvs;       /* number of lvs in vg */
      short                 maxlvs;       /* max number of lvs allowed in vg */
      short                 pp_size;      /* size of pps in the vg */
      short                 numpvs;       /* number of pvs in the vg */
      short                 total_vgdas;  /* number of copies of vg */
					                      /* descriptor area on disk */
      short                 vgda_size;    /* size of volume group descriptor */
      short		    bigvg;
      short		    quorum;
      short		    auto_varyon;
      int		    checksum;
      int		    bigda_size;
   } vg_header;
 
typedef struct _lv_entries
   {
      short       lvname;  	      /* name of LV */
      short       res1;	   	      /* reserved area */
      int        maxsize;   	  /* maximum number of partitions allowed */
      char        lv_state; 	  /* state of logical volume */
      char        mirror;      	  /* none,single, or double */
      short       mirror_policy;  /* type of writing used to write */
      int        num_lps;	      /* number of logical partitions on the lv */
                          	      /* base 1 */
      char        permissions; 	  /* read write or read only */
      char        bb_relocation;  /* specifies if bad block */
                                  /* relocation is desired */
      char        write_verify;   /* verify all writes to the LV */
      char        mirwrt_consist; /* mirror write consistency flag */
      unsigned short  stripe_exp;  /* stripe size in exponent value */
      unsigned short  striping_width;   /* stripe width */
      unsigned short  lv_avoid;
      unsigned short  child_minor_num;
      char      res4[4];           /* reserved area on disk */
   } lv_entries;

 
typedef struct _pv_header
   {
      unique_id             pv_id;      /* unique identifier of PV */
      unsigned short        pp_count;   /* number of physical partitions */
                                        /* on PV */
      char                  pv_state;   /* state of physical volume */
      char                  res1;       /* reserved area on disk */
      daddr_t               psn_part1;  /* physical sector number of 1st pp */
      short                 pvnum_vgdas;/* number of vg descriptor areas */
                                        /* on the physical volume */
      short                 pv_num;     /* PV number */
      long                  res2;     /* reserved area on disk */

    } pv_header;
 
typedef struct _pp_entries
    {
       short        lv_index;     /* index to lv pp is on */
       short        res_1;        /* reserved area on disk */
       long         lp_num;       /* log. part. number */
       char         copy;         /* the copy of the logical partition */
				                  /* that this pp is allocated for */
       char         pp_state;     /* current state of pp */
       char         fst_alt_vol;  /* pv where partition allocation for*/
                                  /* first mirror begins */
       char         snd_alt_vol;  /* pv where partition allocation for*/
                                  /* second mirror begins */ 
       short        fst_alt_part; /* partition to begin first mirror */
       short        snd_alt_part; /*partition to begin second mirror */
       double       res_3;        /* reserved area  on disk */
       double       res_4;        /* reserved area on disk */
    } pp_entries;

typedef struct _namelist
{
   char       name[LVM_MAXLVS][LVM_NAMESIZ];
} namelist;
 
typedef struct _vg_trailer
{
	timestruc_t	    timestamp; /*  time of last update */
	short			concurrency;
	/* MS Nibble = concurrent capable			*/
	/* LS Nibble = concurrent auto-varyon			*/
	short			res_2;
	int			    res_3;	/* reserved area on disk */
	double			res_4;	/* reserved area on disk */
	double			res_5;	/* reserved area on disk */
} vg_trailer;

