/* -*- linux-c -*- */
/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * linux/include/linux/evms_drvlink.h
 *
 * EVMS DriveLink Feature kernel header file
 *
 */

#ifndef __EVMS_DRIVELINK_INCLUDED__
#define __EVMS_DRIVELINK_INCLUDED__

#define EVMS_DRIVELINK_VERSION_MAJOR            2
#define EVMS_DRIVELINK_VERSION_MINOR            0
#define EVMS_DRIVELINK_VERSION_PATCHLEVEL       0

#define EVMS_DRIVELINK_FEATURE_ID       1
#define EVMS_DRIVELINK_SIGNATURE        0x4C767244   //DrvL
#define EVMS_DRIVELINK_MAX_ENTRIES      60

// description of on disk meta data sector for drivelink feature

typedef struct evms_dl_ordering_table_entry_s {
	u_int64_t			child_serial_number;
	evms_sector_t			child_vsize;
} evms_dl_ordering_table_entry_t;

typedef struct evms_drivelink_metadata_s {
/*  0*/ u_int32_t                       signature;
/*  4*/ u_int32_t                       crc;
/*  8*/	evms_version_t			version;
/* 20*/ u_int32_t			flags;
/* 24*/ u_int64_t			sequence_number;
/* 32*/ u_int64_t                       child_serial_number;
/* 40*/ u_int64_t                       parent_serial_number;
/* 48*/ u_int64_t                       child_count;
/* 56*/ u_int64_t			pad;
/* 64*/ evms_dl_ordering_table_entry_t  ordering_table[EVMS_DRIVELINK_MAX_ENTRIES];
/*1024*/
} evms_drivelink_metadata_t;

#ifdef __KERNEL__
// description of in memory meta data for drivelink feature
typedef struct evms_drivelink_runtime_entry_s {
        u_int64_t                       block_size;
        evms_sector_t                   voffset;
        evms_sector_t                   vsize;
        evms_logical_node_t            *child_node;
        evms_drivelink_metadata_t      *child_metadata;
} evms_drivelink_runtime_entry_t;

typedef struct evms_drivelink_runtime_data_s {
        u_int64_t                       block_size;
	// keep the fields below this point in order
        u_int64_t                       parent_serial_number;
        u_int64_t                       child_count;
        evms_drivelink_runtime_entry_t *child_table;
} evms_drivelink_runtime_data_t;
#endif

#endif
