
/* Begin Status/Return value declarations */

enum
{
	GR_ALLOW    =  1,
	GR_NOTFOUND = -1,
	GR_DENY     =  0
};

/* Begin GrSecurity status declarations */

enum
{
	GR_READY    = 0x01,
	GR_1ST      = 0x02,
	GR_LOADING  = 0x04,
	GR_PROCFULL = 0x08,
	/* Out of memory: Process label tables can no
	  longer have entries added*/
	GR_NAMEFULL = 0x10,
	GR_FILEFULL = 0x20,
	GR_STATUS_INIT = GR_1ST
};

/* Table Size Defaults (in the form of 2^GR_TABLE_SIZE). Since
 * our tables are dynamic, this is just the starting point, set to
 * a reasonable value for the predicted number of MAC labels */
enum 
{
	GR_TABLE_SIZE = 8 /* 256 entries by default */
};


/* Begin  ACL declarations */
enum 
{
	GR_READ        = 0x01,
	GR_APPEND      = 0x02,
	GR_WRITE       = 0x04,
	GR_EXEC        = 0x08,
	GR_HIDDEN      = 0x10,
	GR_OVERRIDE    = 0x20, /* Override File ACL, useful for proc only*/
	GR_PROTECTED   = 0x40,
	GR_EXISTS      = 0x80, /* ACL exists....since our hash table 
			  will have blank entries */
	GR_DELETED     = 0x100,
	GR_NONEXISTANT = 0x40000000    /*  fs/namei.c
				   	*  Used to indicate that the file
				   	*  we are opening is nonexistant so we 
			 	   	*  should examine its parent 
					*  for acl rules */
};

/* File Label declarations (right now used in case we need to combine
 * any of preceding flags....like we do for GR_PROC_WRITE below) */
enum
{
	GR_FILE_READ   = GR_READ,
	GR_FILE_APPEND = GR_APPEND,
	GR_FILE_WRITE  = (GR_WRITE | GR_APPEND),
	GR_FILE_EXEC   = GR_EXEC,
	GR_FILE_HIDDEN = GR_HIDDEN
};

/* Process label declarations */

enum
{
	GR_PROC_READ         = GR_READ,
	GR_PROC_APPEND       = GR_APPEND,
	GR_PROC_WRITE        = (GR_WRITE|GR_APPEND),
	/* write access implies append access*/
	GR_PROC_EXEC         = GR_EXEC,
	GR_PROC_ACCESS       = GR_HIDDEN,
	GR_PROC_OVERRIDE     = GR_OVERRIDE,
	GR_PROC_HIDDEN       = GR_HIDDEN,
		/* Hidden, unkillable(except by init) process.
		 * Good for hiding your IDSes / logging daemons:)*/
	GR_PROC_PROTECTED    = GR_PROTECTED,
	GR_INHERIT           = 0x400, 
	/* Children inherit parent's process acls*/
	GR_AUTH              = 0x100,  
	/* Require secondary authentication before execution*/
	GR_AUTH_IF_ROOT      = 0x800,
	/* Require secondary authentication IF the user has root privileges */
	GR_PROC_INHERIT      = GR_INHERIT,
	GR_PROC_AUTH         = GR_AUTH,
	GR_PROC_AUTH_IF_ROOT = GR_AUTH_IF_ROOT,
	GR_PROC_KILL 	     = 0x1000,
	GR_PROC_VIEW         = 0x2000,
	GR_PROC_EXTRA        = (GR_PROC_OVERRIDE|GR_PROC_INHERIT|GR_PROC_AUTH)
	/* Extra things that must be checked after a search*/
};

/* To be implemented : ID based rulesets */

enum
{
	GR_UID            = 0x01,
	GR_GID            = 0x02,
	/* for ID acls(to distinguish between UID and GID) */
	GR_ALL            = 0x04, 
	/* for the 1 UID or GID * ACL the user is allowed to set */
	GR_ID_TPE         = 0x01,
	GR_ID_FORKBOMB    = 0x02,
	GR_ID_PTRACE      = 0x04,
	GR_ID_EXECLOG     = 0x08
};


enum
{
	/* For ID acls using ports. Specifies whether a port 
	 * is for binding or connecting */
	GR_SOCKET_BIND    = 0x01,
	GR_SOCKET_CONNECT = 0x02
};

/* Name hash table declarations. Used when mapping filename->inode/dev
 * and to tell whether a particular inode/dev/filename triplet has 
 * a process subject, process label, and/or file label assigned to it */
enum
{
	GR_NAME_PROC_SUBJECT = 0x01,
	GR_NAME_PROC_OBJECT  = 0x02,
	GR_NAME_FILE         = 0x04
};


/* If we have an acl for a file that did not exist at initialization,
 * set the inode/dev pair to something that will not conflict with
 * anything else (since device 0 is the NULL device we're fine :) */
enum
{
	GR_NOFILE_INODE = 42,
	GR_NOFILE_DEVICE = 0
};

#define PROC_CONF	"/etc/grsec/proc.acl"
#define FILE_CONF	"/etc/grsec/file.acl"
#define PW_CONF		"/etc/grsec/pw"
#define ID_CONF		"/etc/grsec/id.acl"


#define GRADM_ACL           CONFIG_GRADM_PATH " fxi"
#define GODMODE_ACL         "/ frwxoi"

enum 
{
	SET_AUTH     =        1,
	SET_INHERIT  =        2,
	SET_PROC     =        3	
};

enum
{
	PROC_SUBJ_LABEL_OFFSET = 2
/* The first two labels, position 0 and 1, are authentication and god modes
 * respectively, therefore all added process subject labels must begin
 * at offset 2 */
};
